# "Enums"
LED_LEFT = 42
LED_MID = 43
LED_RIGHT = 44
LED_FRONT_LEFT = 45
LED_FRONT_RIGHT = 46
RED = 47
GREEN = 48
YELLOW = 49
ON = 50
OFF = 51
BUTTON_LEFT = 52
BUTTON_MID = 53
BUTTON_RIGHT = 54

# Pin definitions
LED_LEFT_RED_PIN = 19
LED_LEFT_GREEN_PIN = 0

LED_MID_RED_PIN = 5
LED_MID_GREEN_PIN = 6

LED_RIGHT_RED_PIN = 16
LED_RIGHT_GREEN_PIN = 13

LED_FRONT_RIGHT_PIN = 25
LED_FRONT_LEFT_PIN = 26

BUTTON_LEFT_PIN = 1
BUTTON_MID_PIN = 12
BUTTON_RIGHT_PIN = 20
